@@ -287,7 +287,7 @@ KConfigBackend::ParseInfo KConfigIniBackend::parseConfig(const QByteArray &curre
     }
 
     // now make sure immutable groups are marked immutable
-    for (const QByteArray &group : qAsConst(immutableGroups)) {
+    for (const QByteArray &group : std::as_const(immutableGroups)) {
         entryMap.setEntry(group, QByteArray(), QByteArray(), KEntryMap::EntryImmutable);
     }
 
@@ -505,24 +505,19 @@ bool KConfigIniBackend::writeConfig(const QByteArray &locale, KEntryMap &entryMa
         }
     } else {
         // Open existing file. *DON'T* create it if it suddenly does not exist!
-#ifdef Q_OS_UNIX
+#if defined(Q_OS_UNIX) && !defined(Q_OS_ANDROID)
         int fd = QT_OPEN(QFile::encodeName(filePath()).constData(), O_WRONLY | O_TRUNC);
         if (fd < 0) {
             return false;
         }
-        FILE *fp = ::fdopen(fd, "w");
-        if (!fp) {
-            QT_CLOSE(fd);
-            return false;
-        }
         QFile f;
-        if (!f.open(fp, QIODevice::WriteOnly)) {
-            fclose(fp);
+        if (!f.open(fd, QIODevice::WriteOnly)) {
+            QT_CLOSE(fd);
             return false;
         }
         writeEntries(locale, f, writeMap);
         f.close();
-        fclose(fp);
+        QT_CLOSE(fd);
 #else
         QFile f(filePath());
         // XXX This is broken - it DOES create the file if it is suddenly gone.
@@ -539,27 +534,26 @@ bool KConfigIniBackend::writeConfig(const QByteArray &locale, KEntryMap &entryMa
 bool KConfigIniBackend::isWritable() const
 {
     const QString filePath = this->filePath();
-    if (!filePath.isEmpty()) {
-        QFileInfo file(filePath);
-        if (!file.exists()) {
-            // If the file does not exist, check if the deepest
-            // existing dir is writable.
-            QFileInfo dir(file.absolutePath());
-            while (!dir.exists()) {
-                QString parent = dir.absolutePath(); // Go up. Can't use cdUp() on non-existing dirs.
-                if (parent == dir.filePath()) {
-                    // no parent
-                    return false;
-                }
-                dir.setFile(parent);
-            }
-            return dir.isDir() && dir.isWritable();
-        } else {
-            return file.isWritable();
-        }
+    if (filePath.isEmpty()) {
+        return false;
     }
 
-    return false;
+    QFileInfo file(filePath);
+    if (file.exists()) {
+        return file.isWritable();
+    }
+
+    // If the file does not exist, check if the deepest existing dir is writable
+    QFileInfo dir(file.absolutePath());
+    while (!dir.exists()) {
+        QString parent = dir.absolutePath(); // Go up. Can't use cdUp() on non-existing dirs.
+        if (parent == dir.filePath()) {
+            // no parent
+            return false;
+        }
+        dir.setFile(parent);
+    }
+    return dir.isDir() && dir.isWritable();
 }
 
 QString KConfigIniBackend::nonWritableErrorMessage() const
@@ -575,27 +569,28 @@ void KConfigIniBackend::createEnclosing()
     }
 
     // Create the containing dir, maybe it wasn't there
-    QDir dir;
-    dir.mkpath(QFileInfo(file).absolutePath());
+    QDir().mkpath(QFileInfo(file).absolutePath());
 }
 
-void KConfigIniBackend::setFilePath(const QString &file)
+void KConfigIniBackend::setFilePath(const QString &path)
 {
-    if (file.isEmpty()) {
+    if (path.isEmpty()) {
         return;
     }
 
-    Q_ASSERT(QDir::isAbsolutePath(file));
+    Q_ASSERT(QDir::isAbsolutePath(path));
 
-    const QFileInfo info(file);
+    const QFileInfo info(path);
     if (info.exists()) {
         setLocalFilePath(info.canonicalFilePath());
+        return;
+    }
+
+    if (QString filePath = info.dir().canonicalPath(); !filePath.isEmpty()) {
+        filePath += QLatin1Char('/') + info.fileName();
+        setLocalFilePath(filePath);
     } else {
-        const QString dir = info.dir().canonicalPath();
-        if (!dir.isEmpty())
-            setLocalFilePath(dir + QLatin1Char('/') + info.fileName());
-        else
-            setLocalFilePath(file);
+        setLocalFilePath(path);
     }
 }
 
@@ -616,9 +611,25 @@ bool KConfigIniBackend::lock()
 {
     Q_ASSERT(!filePath().isEmpty());
 
+    m_mutex.lock();
+#ifdef Q_OS_ANDROID
+    if (!lockFile) {
+        // handle content Uris properly
+        if (filePath().startsWith(QLatin1String("content://"))) {
+            // we can't create file at an arbitrary location, so use internal storage to create one
+
+            // NOTE: filename can be the same, but because this lock is short lived we may never have a collision
+            lockFile = new QLockFile(QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1String("/")
+                                     + QFileInfo(filePath()).fileName() + QLatin1String(".lock"));
+        } else {
+            lockFile = new QLockFile(filePath() + QLatin1String(".lock"));
+        }
+    }
+#else
     if (!lockFile) {
         lockFile = new QLockFile(filePath() + QLatin1String(".lock"));
     }
+#endif
 
     lockFile->lock();
     return lockFile->isLocked();
@@ -629,6 +640,7 @@ void KConfigIniBackend::unlock()
     lockFile->unlock();
     delete lockFile;
     lockFile = nullptr;
+    m_mutex.unlock();
 }
 
 bool KConfigIniBackend::isLocked() const
